#!/usr/local/bin/perl

#
# Mailback v1.0, an automatic message responder by Derek Vadala
#
#

# User definable parameters
#
# Change this to the path that your mailback files are in. 
# 
$path = "/usr/local/mailback"; 
#
# If you don't want logging then comment out the next line.
#
$logging = "yes"; 
#
# Complete path to the sendmail binary
#
$sendmail = "/usr/sbin/sendmail"; #complete path to sendmail
#
# If you don't want the original message to be copied back to the sender,
# then you should comment out the next line
#
$includeOriginal = "yes";

# No more user-servicable parts below this line.
#

#
# Disintergrate the input record separator so we can slurp our mail and
# config files into one variable for easy splits and output.
#

undef $/;
$incoming = <>;
($inHeader, $inMessage) = split(/\n\n/,$incoming,2);

$time = localtime(time); # grab the time, we'll use it a bit

# Here we pop the headers into an array after our split and grep out
# the important info, TO, FROM and SUBJECT and set up our hash for 
# the sendmail function. NOTE: We swap the To and From headers here
# this saves us a step later.

@headers = split(/\n/,$inHeader);
foreach $header (@headers) {

	($junk, $mail{'Subject'}) = split(/: /,$header) if $header =~ m/^Subject:/;
	($junk, $mail{'From'}) = split(/: /,$header) if $header =~ m/^To:/;
	($junk, $mail{'To'}) = split(/: /,$header) if $header =~ m/^From:/;
	($junk, $mail{'To'}) = split(/: /,$header) if $header =~ m/^Reply-To:/;

}


# Check to see if the recipient has an fqdn in it. If so, we split on @ and
# return the left side, the recipient, to $id for use in the open command which
# follows. This allows us to send out different messages based on recipients.
# So you only need one copy of mailback.

if ($mail{'From'} =~ m/\@/) {

	
	($id,$junk) = split(/\@/,$mail{'From'});
}

else { 

	$id = $mail{'From'};

}

# Now we have enough information to decide if we want to abort, so lets check.
&checkExceptions;
&log("autoresponse") if $logging;

# Open our response message

open MESSAGE, "$path/message-$id";
$mail{'Message'} = <MESSAGE>;
close MESSAGE;



&sendMessage;

# Set up a formatted mail message for sendmail using the hash we've been
# building and then pipe it into sendmail. The opened pipe to sendmail -t 
# comes from some posts on Bugtraq. Net::SMTP might be safer. Next version?

sub sendMessage {


	$sendmailMessage = 	"From: $mail{'From'}\nDate: $time\nTo: $mail{'To'}\nSubject: Re: $mail{'Subject'}\n\n$mail{'Message'}";

	if ($includeOriginal) { 

		$inMessage =~ s/^/>/g;
		$inMessage =~ s/\n/\n>/g;
		$inMessage = "\nYou wrote:\n\n" . $inMessage;
		$sendmailMessage = 	"From: $mail{'From'}\nDate: $time\nTo: $mail{'To'}\nSubject: Re: $mail{'Subject'}\n\n$mail{'Message'}$inMessage";

	}
	else {

		$sendmailMessage = 	"From: $mail{'From'}\nDate: $time\nTo: $mail{'To'}\nSubject: Re: $mail{'Subject'}\n\n$mail{'Message'}";

	}
	
	
	open (MAIL , "|$sendmail -t");
	print MAIL $sendmailMessage;
	print $sendmailMessage;
	close (MAIL);

}

# This subroutine checks to see if we really need to send an auto response.
# A list of exceptions, one per line, is in the exceptions-$id, which like
# the message- file, is generated based on the incoming message.

sub checkExceptions {

	local $realSender = $mail{'To'};
	$realSender =~ s/.*<(.*)>/$1/; # eliminate full names from email address
	open (EXCEPTIONS, "$path/exceptions-$id");
	foreach $person (<EXCEPTIONS>) {
		
		chop($person); # not sure why we chop instead of chomp
		if ($realSender eq $person) {
		
			&log("excepted");
			exit;

		}

	}
}


# Logs a transcript of the autoresponse if we turned on logging.

sub log {

	$action = shift @_;
	open (LOG, ">>$path/log");

	print LOG "$time :  $action to $mail{'To'} for $mail{'From'} regarding $mail{'Subject'}\n";

}
